@@ -19,7 +19,7 @@ from random import randint # make it possible to do testdata.randint so 2 import
 from collections import deque
 import types
 
-__version__ = '0.5.3'
+__version__ = '0.5.8'
 
 def create_file_structure(path_str, tmpdir=u""):
     """
@@ -42,31 +42,38 @@ def create_file_structure(path_str, tmpdir=u""):
     depth_queue.appendleft(0)
     path_prefix = deque()
     path_prefix.appendleft([])
+    current_path = []
 
     for path in path_lines:
         # get the size of the whitespace:
         m = re.match("^\s*", path)
         current_depth = len(m.group(0))
+
         if current_depth > depth_queue[0]:
             depth_queue.appendleft(current_depth)
+            path_prefix.appendleft(current_path)
+
         elif current_depth < depth_queue[0]:
-            depth = depth_queue[0]
-            while current_depth < depth:
-                depth = depth_queue.popleft()
+            while current_depth < depth_queue[0]:
+                depth_queue.popleft()
                 path_prefix.popleft()
 
             if len(path_prefix) == 0:
                 path_prefix.appendleft([])
+
             if len(depth_queue) == 0:
                 depth_queue.appendleft(0)
 
+
         p = path.strip()
         is_dir = p.endswith("/") or p.endswith(os.sep)
         p = _normpath(p)
+
+        #pout.b(p)
+        #pout.v(current_depth, path_prefix, depth_queue)
         if p:
             full_path = filter(None, list(path_prefix[0]) + p.split(os.sep))
             if is_dir:
-                path_prefix.appendleft(full_path)
                 dirs.add(os.sep.join(full_path))
             else:
                 file_path = os.sep.join(full_path)
@@ -74,9 +81,9 @@ def create_file_structure(path_str, tmpdir=u""):
                 files.add(file_path)
                 dirs.add(dir_path)
 
-                full_dir_path = filter(None, dir_path.split(os.sep))
-                if path_prefix[0] != full_dir_path:
-                    path_prefix.appendleft(full_dir_path)
+                full_path = filter(None, dir_path.split(os.sep))
+
+            current_path = full_path
 
     if not tmpdir: tmpdir = tempfile.mkdtemp()
 
@@ -91,7 +98,7 @@ def create_file_structure(path_str, tmpdir=u""):
     return tmpdir, ret_dirs, ret_files
 
 
-def create_dir(path, tmpdir=u""):
+def create_dir(path=u"", tmpdir=u""):
     '''
     create a directory path using a tempdir as the root
 
@@ -145,6 +152,22 @@ def create_file(path, contents=u"", tmpdir=u""):
     return file_path
 
 
+def create_files(file_dict, tmpdir=u""):
+    """
+    create a whole bunch of files all at once
+
+    file_dict -- dict -- keys are the file_name, values are the file contents
+    tmpdir -- string -- same as create_module() tmpdir
+    """
+    ret_files = []
+    base_dir = create_dir(u"", tmpdir)
+
+    for file_name, contents in file_dict.iteritems():
+        ret_files.append(create_file(file_name, contents, base_dir))
+
+    return ret_files
+
+
 def create_modules(module_dict, tmpdir=u"", make_importable=True):
     """
     create a whole bunch of modules all at once
@@ -176,28 +199,28 @@ def create_module(module_name, contents=u"", tmpdir=u"", make_importable=True):
     '''
     module_file = ''
     mod_bits = filter(None, module_name.split(u'.'))
-    module_base_dir = create_dir(u"", tmpdir)
-    base_dir = module_base_dir
-    for modname in mod_bits:
-        base_dir = create_dir(modname, base_dir)
-        module_file = create_file(u"__init__.py", contents=contents, tmpdir=base_dir)
-
-    # add the path to the top of the sys path so importing the new module will work
-    if make_importable:
-        sys.path.insert(0, module_base_dir) 
-
-    return module_file
-
-
-    mod_bits = filter(None, module_name.split(u'.'))
-    module_base_dir = create_dir(u"", tmpdir)
+    module_base_dir = create_dir(u"", tmpdir=tmpdir)
     base_modname = mod_bits.pop()
+
     base_dir = module_base_dir
     for modname in mod_bits:
-        base_dir = create_dir(modname, base_dir)
-        create_file(u"__init__.py", tmpdir=base_dir)
+        # check to see if there is a file that already exists
+        mod_file = os.path.join(base_dir, u"{}.py".format(modname))
 
-    module_file = create_file(u"{}.py".format(base_modname), contents=contents, tmpdir=base_dir)
+        # turn module.py into a package (module/__init__.py)
+        base_dir = create_dir(modname, base_dir)
+        if os.path.isfile(mod_file):
+            os.rename(mod_file, os.path.join(base_dir, u"__init__.py"))
+        else:
+            # only add a blank package sentinel file if one already doesn't exist
+            if not os.path.isfile(os.path.join(base_dir, u"__init__.py")):
+                create_file(u"__init__.py", tmpdir=base_dir)
+
+    mod_dir = os.path.join(base_dir, base_modname)
+    if os.path.isdir(mod_dir):
+        module_file = create_file(u"__init__.py", contents=contents, tmpdir=mod_dir)
+    else:
+        module_file = create_file(u"{}.py".format(base_modname), contents=contents, tmpdir=base_dir)
 
     # add the path to the top of the sys path so importing the new module will work
     if make_importable:
@@ -205,6 +228,7 @@ def create_module(module_name, contents=u"", tmpdir=u"", make_importable=True):
 
     return module_file
 
+
 def get_url():
     '''
     get a url, this is just a nice shortcut method to something I seemed to do a lot
@@ -215,7 +239,8 @@ def get_url():
         u's' if random.choice([True, False]) else u'',
         get_ascii()
     )
-     
+
+
 def get_str(str_size=0, chars=None):
     '''
     generate a random unicode string
@@ -1113,7 +1138,7 @@ def _normpath(path):
 
     for some reason, os.path.split() wouldn't work with the windows slash (\)
     '''
-    if not path: return path
+    if not path: return u''
 
     path = os.path.normpath(path)
     #dirs = filter(None, re.split(ur'[\\/]+', path))
